##############################################
# R script illustrating the use of inter-
# action terms in RSiena; builds on results
# of "lab-coevolution.R".
# version of January 15, 2019
# written by Christian Steglich
##############################################

# Suppose you are interested in the difference
# between what happens in reciprocated friendship
# dyads in comparison to unreciprocated,
# unilateral friendship.
# Then you would want to include interaction
# terms with reciprocity. We do this now for
# two effects: alcohol-based homophily
# (selection) and friends influence on alcohol
# consumption.

# first load namespace:
library(RSiena)

# create sienaData object:
CoEvolutionData <- sienaDataCreate(
	friendship = sienaDependent(
		array(c(s501,s502,s503),dim=c(50,50,3))
	),
	drinking = sienaDependent(s50a,type="behavior")
)

# specify a model:
ModerationModel <- getEffects(CoEvolutionData)
ModerationModel <- includeEffects(ModerationModel,simX,
	interaction1="drinking",name="friendship")
ModerationModel <- includeEffects(ModerationModel,totSim,
	interaction1="friendship",name="drinking")
ModerationModel <- includeEffects(ModerationModel,recip,transTrip,
	transRecTrip,name="friendship")
# full model specification looks like this now:
ModerationModel


# include interaction terms of alcohol homophily with reciprocity:
?includeInteraction
ModerationModel <- includeInteraction(ModerationModel,recip,simX,
	interaction1=c("","drinking"),name="friendship")

# also include interaction of peer influence and reciprocity,
# noting that this effect is pre-programmed:
ModerationModel <- includeEffects(ModerationModel,totSimRecip,
	interaction1="friendship",name="drinking")

# How does the model specification look like?
ModerationModel

# create an object with options for algorithm tuning:
CoEvolutionOptions <- sienaAlgorithmCreate(useStdInits=FALSE,n3=2500,
	projname='CoEvol-results',MaxDegree=c(friendship=6),seed=786840)

# estimate the new model specification (with GUI shown):
ModerationResults <- siena07(CoEvolutionOptions,data=CoEvolutionData,
  effects=ModerationModel,batch=FALSE,verbose=TRUE,
  useCluster=TRUE,nbrNodes=2)
ModerationResults # converged

# At first sight, it looks like no significant results remain here;
# probably there still is a marginally significant main effect of
# drinking homophily in the unreciprocated friendship case, but for
# reciprocal friendships, the effect seems non-existent.
# For peer influence, everything looks non-significant, but
# parameter estimates suggest that influence happens in reciprocal
# dyads rather than in uncreciprocated ones.

# Significance assessment requires more exactl calculations, let us do this.

# For calculation ease, make estimates and covariance matrix of
# estimates available in simpler syntax as "theta" and "covtheta":
attach(ModerationResults)

# Statistically, the main effects can be tested right away; 
# let us calculate approximate p-values (2-sided)
# ... of homophily in unreciprocated frienships:
2*pnorm(theta[7]/sqrt(covtheta[7,7]),lower.tail=FALSE)
# ... and of influence in reciprocal friendships:
2*pnorm(theta[14]/sqrt(covtheta[14,14]),lower.tail=FALSE)
# both non-significant.

# For the combined effects, we need to add or subtract the different
# effects and determine the effect sum's significance. This is a bit
# complicated because we need to calculate the standard error of this
# effect sum making use of the covariance matrix; the underlying
# formula being var(A+B)=var(A)+var(B)+2cov(A,B)...

# Noting that...
#    > reciprocity has effect number 4
#    > homophily has effect number 7
#    > etc.
# we get the following calculation:

# calculate joint effect of reciprocity and homophily: 
hrec <- theta[4]+theta[7]+theta[8] # sum of main effects and interaction terms
# calculate st.error of this effect sum:
se.hrec <- sqrt(covtheta[4,4]+covtheta[7,7]+covtheta[8,8]+
	2*covtheta[4,7]+2*covtheta[4,8]+2*covtheta[7,8])
# calculate p-value (2-sided):
2*pnorm(hrec/sqrt(se.hrec),lower.tail=FALSE)
# highly significant (but mainly due to strong reciprocity).

# now let us assess influence in unreciprocated friendships:
infasy <- theta[13]-theta[14]
se.infasy <- sqrt(covtheta[13,13]+covtheta[14,14]-2*covtheta[13,14])
2*pnorm(infasy/sqrt(se.infasy),lower.tail=TRUE) # note negative effect
# once more non-significant

# Apparently the joint effect of homophily and reciprocity is significant
# - but this is probably largely caused by the main reciprocity effect.

# What we may be interested in is whether this effect is stronger than
# reciprocity without homophily... for this purpose, we'd need to drop
# the contributions of effect #4 (reciprocity) in the above calculation.

hnorec <- theta[7]+theta[8]
se.hnr <- sqrt(covtheta[7,7]+covtheta[8,8]+2*covtheta[7,8])
2*pnorm(hnorec/sqrt(se.hnr),lower.tail=FALSE)
# again n.s.

# Conclusion: reciprocation does not significantly moderate (a) homophily
# on alcohol in friendship dyads, nor (b) peers' influence.

# HOWEVER, the estimated influence effects for unreciprocated vs. reciprocated
# friendships have non-negligible size. If we plot them in a usual
# moderation plot, we get:

simmean <- attr(CoEvolutionData$depvars$drinking,"simMeans")
plot(c(0.2,.8),theta[14]*((0:1)-simmean),
	xlim=c(0,1),ylim=c(-1.5,1.5),
	xaxt='n',xlab='',ylab='objective function value',type='l',col='red',
	main='influence moderated by reciprocity')
lines(c(0.2,.8),infasy*((0:1)-simmean),type='l',col='blue')
axis(1,at=c(0.2,.8),labels=c('sim=0','sim=1'))
legend(0.1,0.2,legend=c('reciprocated','unreciprocated'),text.col=c('red','blue'))

# if we neglect lacking significance, this suggests: 
# girls strive to be similar to other girls that the are reciprocally linked to
# they strive to be different from girls that they are asymmetrically linked to
